@@ -46,12 +46,14 @@
 */
 
 #include "includes.h"
+#include "lib/util/server_id.h"
 #include "dbwrap/dbwrap.h"
 #include "serverid.h"
 #include "messages.h"
 #include "lib/util/tevent_unix.h"
 #include "lib/background.h"
 #include "lib/messages_dgm.h"
+#include "lib/messages_ctdbd.h"
 #include "lib/util/iov_buf.h"
 #include "lib/util/server_id_db.h"
 #include "lib/messages_dgm_ref.h"
@@ -72,10 +74,10 @@ struct messaging_context {
 	struct messaging_callback *callbacks;
 
 	struct tevent_req **new_waiters;
-	unsigned num_new_waiters;
+	size_t num_new_waiters;
 
 	struct tevent_req **waiters;
-	unsigned num_waiters;
+	size_t num_waiters;
 
 	void *msg_dgm_ref;
 	struct messaging_backend *remote;
@@ -108,7 +110,7 @@ static void ping_message(struct messaging_context *msg_ctx,
 	messaging_send(msg_ctx, src, MSG_PONG, data);
 }
 
-static struct messaging_rec *messaging_rec_create(
+struct messaging_rec *messaging_rec_create(
 	TALLOC_CTX *mem_ctx, struct server_id src, struct server_id dst,
 	uint32_t msg_type, const struct iovec *iov, int iovlen,
 	const int *fds, size_t num_fds)
@@ -211,7 +213,7 @@ close_fail:
 
 static int messaging_context_destructor(struct messaging_context *ctx)
 {
-	unsigned i;
+	size_t i;
 
 	for (i=0; i<ctx->num_new_waiters; i++) {
 		if (ctx->new_waiters[i] != NULL) {
@@ -246,6 +248,13 @@ static NTSTATUS messaging_init_internal(TALLOC_CTX *mem_ctx,
 	const char *priv_path;
 	bool ok;
 
+	/*
+	 * sec_init() *must* be called before any other
+	 * functions that use sec_XXX(). e.g. sec_initial_uid().
+	 */
+
+	sec_init();
+
 	lck_path = lock_path("msg.lock");
 	if (lck_path == NULL) {
 		return NT_STATUS_NO_MEMORY;
@@ -290,8 +299,6 @@ static NTSTATUS messaging_init_internal(TALLOC_CTX *mem_ctx,
 
 	ctx->event_ctx = ev;
 
-	sec_init();
-
 	ctx->msg_dgm_ref = messaging_dgm_ref(ctx,
 					     ctx->event_ctx,
 					     &ctx->id.unique_id,
@@ -767,7 +774,7 @@ static void messaging_filtered_read_cleanup(struct tevent_req *req,
 	struct messaging_filtered_read_state *state = tevent_req_data(
 		req, struct messaging_filtered_read_state);
 	struct messaging_context *msg_ctx = state->msg_ctx;
-	unsigned i;
+	size_t i;
 
 	tevent_req_set_cleanup_fn(req, NULL);
 
@@ -904,87 +911,6 @@ int messaging_read_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 	return 0;
 }
 
-struct messaging_handler_state {
-	struct tevent_context *ev;
-	struct messaging_context *msg_ctx;
-	uint32_t msg_type;
-	bool (*handler)(struct messaging_context *msg_ctx,
-			struct messaging_rec **rec, void *private_data);
-	void *private_data;
-};
-
-static void messaging_handler_got_msg(struct tevent_req *subreq);
-
-struct tevent_req *messaging_handler_send(
-	TALLOC_CTX *mem_ctx, struct tevent_context *ev,
-	struct messaging_context *msg_ctx, uint32_t msg_type,
-	bool (*handler)(struct messaging_context *msg_ctx,
-			struct messaging_rec **rec, void *private_data),
-	void *private_data)
-{
-	struct tevent_req *req, *subreq;
-	struct messaging_handler_state *state;
-
-	req = tevent_req_create(mem_ctx, &state,
-				struct messaging_handler_state);
-	if (req == NULL) {
-		return NULL;
-	}
-	state->ev = ev;
-	state->msg_ctx = msg_ctx;
-	state->msg_type = msg_type;
-	state->handler = handler;
-	state->private_data = private_data;
-
-	subreq = messaging_read_send(state, state->ev, state->msg_ctx,
-				     state->msg_type);
-	if (tevent_req_nomem(subreq, req)) {
-		return tevent_req_post(req, ev);
-	}
-	tevent_req_set_callback(subreq, messaging_handler_got_msg, req);
-	return req;
-}
-
-static void messaging_handler_got_msg(struct tevent_req *subreq)
-{
-	struct tevent_req *req = tevent_req_callback_data(
-		subreq, struct tevent_req);
-	struct messaging_handler_state *state = tevent_req_data(
-		req, struct messaging_handler_state);
-	struct messaging_rec *rec;
-	int ret;
-	bool ok;
-
-	ret = messaging_read_recv(subreq, state, &rec);
-	TALLOC_FREE(subreq);
-	if (tevent_req_error(req, ret)) {
-		return;
-	}
-
-	subreq = messaging_read_send(state, state->ev, state->msg_ctx,
-				     state->msg_type);
-	if (tevent_req_nomem(subreq, req)) {
-		return;
-	}
-	tevent_req_set_callback(subreq, messaging_handler_got_msg, req);
-
-	ok = state->handler(state->msg_ctx, &rec, state->private_data);
-	TALLOC_FREE(rec);
-	if (ok) {
-		/*
-		 * Next round
-		 */
-		return;
-	}
-	TALLOC_FREE(subreq);
-	tevent_req_done(req);
-}
-
-int messaging_handler_recv(struct tevent_req *req)
-{
-	return tevent_req_simple_recv_unix(req);
-}
-
 static bool messaging_append_new_waiters(struct messaging_context *msg_ctx)
 {
 	if (msg_ctx->num_new_waiters == 0) {
@@ -1013,7 +939,7 @@ static bool messaging_append_new_waiters(struct messaging_context *msg_ctx)
 	return true;
 }
 
-static void messaging_dispatch_classic(struct messaging_context *msg_ctx,
+static bool messaging_dispatch_classic(struct messaging_context *msg_ctx,
 				       struct messaging_rec *rec)
 {
 	struct messaging_callback *cb, *next;
@@ -1039,13 +965,10 @@ static void messaging_dispatch_classic(struct messaging_context *msg_ctx,
 		cb->fn(msg_ctx, cb->private_data, rec->msg_type,
 		       rec->src, &rec->buf);
 
-		/*
-		 * we continue looking for matching messages after finding
-		 * one. This matters for subsystems like the internal notify
-		 * code which register more than one handler for the same
-		 * message type
-		 */
+		return true;
 	}
+
+	return false;
 }
 
 /*
@@ -1055,14 +978,18 @@ static void messaging_dispatch_rec(struct messaging_context *msg_ctx,
 				   struct tevent_context *ev,
 				   struct messaging_rec *rec)
 {
-	unsigned i;
-	size_t j;
+	size_t i;
+	bool consumed;
 
 	if (ev == msg_ctx->event_ctx) {
-		messaging_dispatch_classic(msg_ctx, rec);
+		consumed = messaging_dispatch_classic(msg_ctx, rec);
+		if (consumed) {
+			return;
+		}
 	}
 
 	if (!messaging_append_new_waiters(msg_ctx)) {
+		size_t j;
 		for (j=0; j < rec->num_fds; j++) {
 			int fd = rec->fds[j];
 			close(fd);
@@ -1100,12 +1027,7 @@ static void messaging_dispatch_rec(struct messaging_context *msg_ctx,
 		if ((ev == state->ev) &&
 		    state->filter(rec, state->private_data)) {
 			messaging_filtered_read_done(req, rec);
-
-			/*
-			 * Only the first one gets the fd-array
-			 */
-			rec->num_fds = 0;
-			rec->fds = NULL;
+			return;
 		}
 
 		i += 1;
@@ -1140,8 +1062,8 @@ static void messaging_dispatch_rec(struct messaging_context *msg_ctx,
 	/*
 	 * If the fd-array isn't used, just close it.
 	 */
-	for (j=0; j < rec->num_fds; j++) {
-		int fd = rec->fds[j];
+	for (i=0; i < rec->num_fds; i++) {
+		int fd = rec->fds[i];
 		close(fd);
 	}
 	rec->num_fds = 0;
