@@ -37,6 +37,7 @@
 
 #include "includes.h"
 #include "system/filesys.h"
+#include "lib/util/server_id.h"
 #include "locking/proto.h"
 #include "smbd/globals.h"
 #include "dbwrap/dbwrap.h"
@@ -49,6 +50,7 @@
 #include "source3/lib/dbwrap/dbwrap_watch.h"
 #include "locking/leases_db.h"
 #include "../lib/util/memcache.h"
+#include "lib/util/tevent_ntstatus.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_LOCKING
@@ -322,8 +324,8 @@ static struct share_mode_data *parse_share_modes(TALLOC_CTX *mem_ctx,
 	}
 
 	/*
-	 * Initialize the values that are [skip] in the idl. The NDR code does
-	 * not initialize them.
+	 * Initialize the values that are [skip] or [ignore]
+	 * in the idl. The NDR code does not initialize them.
 	 */
 
 	for (i=0; i<d->num_share_modes; i++) {
@@ -622,19 +624,28 @@ fail:
 	return NULL;
 }
 
+struct fetch_share_mode_unlocked_state {
+	TALLOC_CTX *mem_ctx;
+	struct share_mode_lock *lck;
+};
+
 static void fetch_share_mode_unlocked_parser(
 	TDB_DATA key, TDB_DATA data, void *private_data)
 {
-	struct share_mode_lock *lck = talloc_get_type_abort(
-		private_data, struct share_mode_lock);
+	struct fetch_share_mode_unlocked_state *state = private_data;
 
 	if (data.dsize == 0) {
 		/* Likely a ctdb tombstone record, ignore it */
-		lck->data = NULL;
 		return;
 	}
 
-	lck->data = parse_share_modes(lck, key, data);
+	state->lck = talloc(state->mem_ctx, struct share_mode_lock);
+	if (state->lck == NULL) {
+		DEBUG(0, ("talloc failed\n"));
+		return;
+	}
+
+	state->lck->data = parse_share_modes(state->lck, key, data);
 }
 
 /*******************************************************************
@@ -645,23 +656,137 @@ static void fetch_share_mode_unlocked_parser(
 struct share_mode_lock *fetch_share_mode_unlocked(TALLOC_CTX *mem_ctx,
 						  struct file_id id)
 {
-	struct share_mode_lock *lck;
+	struct fetch_share_mode_unlocked_state state = { .mem_ctx = mem_ctx };
 	TDB_DATA key = locking_key(&id);
 	NTSTATUS status;
 
-	lck = talloc(mem_ctx, struct share_mode_lock);
-	if (lck == NULL) {
-		DEBUG(0, ("talloc failed\n"));
-		return NULL;
-	}
 	status = dbwrap_parse_record(
-		lock_db, key, fetch_share_mode_unlocked_parser, lck);
-	if (!NT_STATUS_IS_OK(status) ||
-	    (lck->data == NULL)) {
-		TALLOC_FREE(lck);
+		lock_db, key, fetch_share_mode_unlocked_parser, &state);
+	if (!NT_STATUS_IS_OK(status)) {
 		return NULL;
 	}
-	return lck;
+	return state.lck;
+}
+
+static void fetch_share_mode_done(struct tevent_req *subreq);
+
+struct fetch_share_mode_state {
+	struct file_id id;
+	TDB_DATA key;
+	struct fetch_share_mode_unlocked_state parser_state;
+	enum dbwrap_req_state req_state;
+};
+
+/**
+ * @brief Get a share_mode_lock without locking or refcounting
+ *
+ * This can be used in a clustered Samba environment where the async dbwrap
+ * request is sent over a socket to the local ctdbd. If the send queue is full
+ * and the caller was issuing multiple async dbwrap requests in a loop, the
+ * caller knows it's probably time to stop sending requests for now and try
+ * again later.
+ *
+ * @param[in]  mem_ctx The talloc memory context to use.
+ *
+ * @param[in]  ev      The event context to work on.
+ *
+ * @param[in]  id      The file id for the locking.tdb key
+ *
+ * @param[out] queued  This boolean out parameter tells the caller whether the
+ *                     async request is blocked in a full send queue:
+ *
+ *                     false := request is dispatched
+ *
+ *                     true  := send queue is full, request waiting to be
+ *                              dispatched
+ *
+ * @return             The new async request, NULL on error.
+ **/
+struct tevent_req *fetch_share_mode_send(TALLOC_CTX *mem_ctx,
+					 struct tevent_context *ev,
+					 struct file_id id,
+					 bool *queued)
+{
+	struct tevent_req *req = NULL;
+	struct fetch_share_mode_state *state = NULL;
+	struct tevent_req *subreq = NULL;
+
+	*queued = false;
+
+	req = tevent_req_create(mem_ctx, &state,
+				struct fetch_share_mode_state);
+	if (req == NULL) {
+		return NULL;
+	}
+
+	state->id = id;
+	state->key = locking_key(&state->id);
+	state->parser_state.mem_ctx = state;
+
+	subreq = dbwrap_parse_record_send(state,
+					  ev,
+					  lock_db,
+					  state->key,
+					  fetch_share_mode_unlocked_parser,
+					  &state->parser_state,
+					  &state->req_state);
+	if (tevent_req_nomem(subreq, req)) {
+		return tevent_req_post(req, ev);
+	}
+	tevent_req_set_callback(subreq, fetch_share_mode_done, req);
+
+	if (state->req_state < DBWRAP_REQ_DISPATCHED) {
+		*queued = true;
+	}
+	return req;
+}
+
+static void fetch_share_mode_done(struct tevent_req *subreq)
+{
+	struct tevent_req *req = tevent_req_callback_data(
+		subreq, struct tevent_req);
+	NTSTATUS status;
+
+	status = dbwrap_parse_record_recv(subreq);
+	TALLOC_FREE(subreq);
+	if (tevent_req_nterror(req, status)) {
+		return;
+	}
+
+	tevent_req_done(req);
+	return;
+}
+
+NTSTATUS fetch_share_mode_recv(struct tevent_req *req,
+			       TALLOC_CTX *mem_ctx,
+			       struct share_mode_lock **_lck)
+{
+	struct fetch_share_mode_state *state = tevent_req_data(
+		req, struct fetch_share_mode_state);
+	struct share_mode_lock *lck = NULL;
+
+	NTSTATUS status;
+
+	if (tevent_req_is_nterror(req, &status)) {
+		tevent_req_received(req);
+		return status;
+	}
+
+	if (state->parser_state.lck->data == NULL) {
+		tevent_req_received(req);
+		return NT_STATUS_NOT_FOUND;
+	}
+
+	lck = talloc_move(mem_ctx, &state->parser_state.lck);
+
+	if (DEBUGLEVEL >= 10) {
+		DBG_DEBUG("share_mode_data:\n");
+		NDR_PRINT_DEBUG(share_mode_data, lck->data);
+	}
+
+	*_lck = lck;
+	tevent_req_received(req);
+	return NT_STATUS_OK;
 }
 
 struct share_mode_forall_state {
