@@ -92,13 +92,16 @@
 #define DEFAULT_QUOTA_GID_NOLIMIT(handle) \
 	lp_parm_bool(SNUM((handle)->conn),DEFAULT_QUOTA_NAME,"gid nolimit",DEFAULT_QUOTA_GID_NOLIMIT_DEFAULT)
 
-static int default_quota_get_quota(vfs_handle_struct *handle, const char *path,
-				   enum SMB_QUOTA_TYPE qtype, unid_t id,
-				   SMB_DISK_QUOTA *dq)
+static int default_quota_get_quota(vfs_handle_struct *handle,
+				const struct smb_filename *smb_fname,
+				enum SMB_QUOTA_TYPE qtype,
+				unid_t id,
+				SMB_DISK_QUOTA *dq)
 {
 	int ret = -1;
 
-	if ((ret = SMB_VFS_NEXT_GET_QUOTA(handle, path, qtype, id, dq)) != 0) {
+	if ((ret = SMB_VFS_NEXT_GET_QUOTA(handle, smb_fname,
+				qtype, id, dq)) != 0) {
 		return ret;
 	}
 
@@ -124,8 +127,8 @@ static int default_quota_get_quota(vfs_handle_struct *handle, const char *path,
 				unid_t qid;
 				uint32_t qflags = dq->qflags;
 				qid.uid = DEFAULT_QUOTA_UID(handle);
-				SMB_VFS_NEXT_GET_QUOTA(
-				    handle, path, SMB_USER_QUOTA_TYPE, qid, dq);
+				SMB_VFS_NEXT_GET_QUOTA(handle, smb_fname,
+					SMB_USER_QUOTA_TYPE, qid, dq);
 				dq->qflags = qflags;
 			}
 			break;
@@ -135,7 +138,7 @@ static int default_quota_get_quota(vfs_handle_struct *handle, const char *path,
 				unid_t qid;
 				uint32_t qflags = dq->qflags;
 				qid.gid = DEFAULT_QUOTA_GID(handle);
-				SMB_VFS_NEXT_GET_QUOTA(handle, path,
+				SMB_VFS_NEXT_GET_QUOTA(handle, smb_fname,
 						       SMB_GROUP_QUOTA_TYPE,
 						       qid, dq);
 				dq->qflags = qflags;
@@ -225,8 +228,8 @@ static struct vfs_fn_pointers vfs_default_quota_fns = {
 	.set_quota_fn = default_quota_set_quota
 };
 
-NTSTATUS vfs_default_quota_init(void);
-NTSTATUS vfs_default_quota_init(void)
+NTSTATUS vfs_default_quota_init(TALLOC_CTX *);
+NTSTATUS vfs_default_quota_init(TALLOC_CTX *ctx)
 {
 	return smb_register_vfs(SMB_VFS_INTERFACE_VERSION, DEFAULT_QUOTA_NAME,
 				&vfs_default_quota_fns);
