@@ -21,6 +21,7 @@
 #include "includes.h"
 #include "system/filesys.h"
 #include "smbd/smbd.h"
+#include "lib/util/sys_rw.h"
 #include "lib/util/sys_rw_data.h"
 
 struct preopen_state;
@@ -185,7 +186,7 @@ static bool preopen_helper_open_one(int sock_fd, char **pnamebuf,
 	close(fd);
 
  done:
-	(void)write(sock_fd, &c, 1);
+	sys_write_v(sock_fd, &c, 1);
 	return true;
 }
 
@@ -407,7 +408,8 @@ static int preopen_open(vfs_handle_struct *handle,
 
 	TALLOC_FREE(state->template_fname);
 	state->template_fname = talloc_asprintf(
-		state, "%s/%s", fsp->conn->cwd, smb_fname->base_name);
+		state, "%s/%s",
+		fsp->conn->cwd_fname->base_name, smb_fname->base_name);
 
 	if (state->template_fname == NULL) {
 		return res;
@@ -448,8 +450,8 @@ static struct vfs_fn_pointers vfs_preopen_fns = {
 	.open_fn = preopen_open
 };
 
-NTSTATUS vfs_preopen_init(void);
-NTSTATUS vfs_preopen_init(void)
+NTSTATUS vfs_preopen_init(TALLOC_CTX *);
+NTSTATUS vfs_preopen_init(TALLOC_CTX *ctx)
 {
 	return smb_register_vfs(SMB_VFS_INTERFACE_VERSION,
 				"preopen", &vfs_preopen_fns);
