@@ -143,11 +143,7 @@ bool reload_services(struct smbd_server_connection *sconn,
 
 	lp_killunused(sconn, snumused);
 
-	ret = lp_load(get_dyn_CONFIGFILE(),
-		      false, /* global only */
-		      false, /* save defaults */
-		      true,  /* add_ipc */
-		      true); /* initialize globals */
+	ret = lp_load_with_shares(get_dyn_CONFIGFILE());
 
 	/* perhaps the config filename is now set */
 	if (!test) {
@@ -168,6 +164,7 @@ bool reload_services(struct smbd_server_connection *sconn,
 
 	mangle_reset_cache();
 	reset_stat_cache();
+	flush_dfree_cache();
 
 	/* this forces service parameters to be flushed */
 	set_current_service(NULL,0,True);
