@@ -29,6 +29,9 @@
 #include "source3/lib/dbwrap/dbwrap_watch.h"
 #include "messages.h"
 
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_SMB2
+
 static struct tevent_req *smbd_smb2_setinfo_send(TALLOC_CTX *mem_ctx,
 						 struct tevent_context *ev,
 						 struct smbd_smb2_request *smb2req,
@@ -248,11 +251,11 @@ static struct tevent_req *delay_rename_for_lease_break(struct tevent_req *req,
 
 	talloc_set_destructor(rename_state, defer_rename_state_destructor);
 
-	subreq = dbwrap_record_watch_send(
+	subreq = dbwrap_watched_watch_send(
 				rename_state,
 				ev,
 				lck->data->record,
-				fsp->conn->sconn->msg_ctx);
+				(struct server_id){0});
 
 	if (subreq == NULL) {
 		exit_server("Could not watch share mode record for rename\n");
@@ -279,7 +282,8 @@ static void defer_rename_done(struct tevent_req *subreq)
 	int ret_size = 0;
 	bool ok;
 
-	status = dbwrap_record_watch_recv(subreq, state->req, NULL);
+	status = dbwrap_watched_watch_recv(subreq, state->req, NULL, NULL,
+					   NULL);
 	TALLOC_FREE(subreq);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(5, ("dbwrap_record_watch_recv returned %s\n",
@@ -298,7 +302,6 @@ static void defer_rename_done(struct tevent_req *subreq)
 		return;
 	}
 
-	/* should we pass FLAG_CASELESS_PATHNAMES here? */
 	ok = set_current_service(state->smb2req->tcon->compat, 0, true);
 	if (!ok) {
 		tevent_req_nterror(state->req, NT_STATUS_ACCESS_DENIED);
@@ -483,6 +486,7 @@ static struct tevent_req *smbd_smb2_setinfo_send(TALLOC_CTX *mem_ctx,
 			lck = get_existing_share_mode_lock(mem_ctx,
 							fsp->file_id);
 			if (lck == NULL) {
+				SAFE_FREE(data);
 				tevent_req_nterror(req,
 					NT_STATUS_UNSUCCESSFUL);
 				return tevent_req_post(req, ev);
@@ -529,6 +533,24 @@ static struct tevent_req *smbd_smb2_setinfo_send(TALLOC_CTX *mem_ctx,
 		break;
 	}
 
+	case 0x02:/* SMB2_SETINFO_FS */
+	{
+		uint16_t file_info_level = in_file_info_class + 1000;
+
+		status = smbd_do_setfsinfo(conn, smbreq, state,
+					file_info_level,
+					fsp,
+					&in_input_buffer);
+		if (!NT_STATUS_IS_OK(status)) {
+			if (NT_STATUS_EQUAL(status, NT_STATUS_INVALID_LEVEL)) {
+				status = NT_STATUS_INVALID_INFO_CLASS;
+			}
+			tevent_req_nterror(req, status);
+			return tevent_req_post(req, ev);
+		}
+		break;
+	}
+
 	case 0x03:/* SMB2_SETINFO_SECURITY */
 	{
 		if (!CAN_WRITE(conn)) {
